require 'open3'

# Runs cobol programs
class CobolRunner
  def initialize(config_file, library, logger)
    @config_file = config_file
    @library = library
    @logger = logger
  end

  def run(acu, args)
    options = %W[
      -b -c #{@config_file} -y #{@library} #{acu}
    ]
    args.map!(&:to_s)
    stdout, stderr, status = Open3.capture3('crun32', *options, *args)

    error = extract_error stdout, stderr, status
    raise StandardError, error if error

    stdout.chomp
  end

  private

  def extract_error(stdout, stderr, status)
    return false if status.success?
    if stdout.empty?
      stderr.chomp
    else
      data = JSON.parse(stdout)
      data.is_a?(Array) ? false : data['result'].chomp
    end
  end
end
