begin
  require 'rubygems'
  require 'win32/daemon'
  require 'rack'
  require 'puma'
  require 'puma/configuration'
  require 'dotenv'
  require 'json'
  include Win32

  APPLICATION_ROOT = File.expand_path(File.join(__dir__, ".."))

  class RxUniverseDaemon < Daemon
    def service_init
      Dir.chdir(APPLICATION_ROOT) do
        Dotenv.load
        p "Initializing service with root #{APPLICATION_ROOT}"
      end
      @puma = Puma::Launcher.new(Puma::Configuration.new do |c|
        c.threads 16, 32
        c.directory APPLICATION_ROOT
        c.environment "production"

        port = ENV['PORT'] || 4567
        ssl_key_path = ENV['SSL_KEY']
        ssl_certificate_path = ENV['SSL_CERTIFICATE']
        if ssl_key_path && ssl_certificate_path
          c.bind "ssl://0.0.0.0:#{port}?key=#{ssl_key_path}&cert=#{ssl_certificate_path}"
        else
          c.bind "tcp://0.0.0.0:#{port}"
        end
      end, events: Puma::Events.stdio)
    end

    def service_stop
      @puma.halt
    end

    def service_main
      @puma.run
    end

    def service_pause
      @puma.stop
    end

    def service_resume
      @puma.run
    end
  end

  RxUniverseDaemon.mainloop
rescue StandardError => err
  begin
    Dir.chdir(APPLICATION_ROOT) do
      File.open("./service-log.txt", 'w') do |f|
        f.puts Time.now
        f.puts err.message
        f.puts err.backtrace.join("\n\t")
      end
    end
  rescue StandardError => dirErr
    Dir.chdir(__dir__) do
      File.open("./service-log.txt", 'w') do |f|
        f.puts Time.now
        f.puts err.message
        f.puts err.backtrace.join("\n\t")
      end
    end
  end
end